import React from "react"
import { Provider as ProviderX } from 'react-redux'
import { RootStore, AppPersistor } from "./config";
import { PersistGate } from 'redux-persist/integration/react'
import WebStorage from 'redux-persist/lib/storage'
import NativeStorage from '@react-native-community/async-storage'
import { xSetState, getStateForKey } from "./methods";

let persisStorage = WebStorage

const stateInitalizer = (initialState: object, isNative: boolean) => {
    if (isNative) { persisStorage = NativeStorage }
    const didInit = getStateForKey('didInit')
    if (!didInit) { xSetState({ ...initialState, didInit: true }) }
}

/** 
 * Step-Ract-Redux Provider
 * @example
 * const myInitialState = { 
 *    // initial state 
 * }
 * <Provider
 *  initialState={myInitialState}
 *  loading={
 *  // your loading UI *
 *  }
 *  >
 *   <App />
 * </Provider>
 * @param {object} initialState
*/
const Provider = ({
    initialState = {},
    isNative = false,
    loading,
    children
}) => {
    stateInitalizer(initialState, isNative)
    return (
        <ProviderX store={RootStore}>
            <PersistGate
                persistor={AppPersistor}
                loading={loading}
                children={children}
            />
        </ProviderX>
    );
}

export {
    Provider,
    persisStorage
}